"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const log_1 = require("@dash4/log");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const execa_1 = tslib_1.__importDefault(require("execa"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const path_1 = tslib_1.__importDefault(require("path"));
const read_pkg_1 = tslib_1.__importDefault(require("read-pkg"));
const write_pkg_1 = tslib_1.__importDefault(require("write-pkg"));
const Config_1 = require("./Config/Config");
const get_lerna_packages_1 = require("./get-lerna-packages");
function hasFile(...pathFragments) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            return (yield fs_extra_1.default.stat(path_1.default.join(...pathFragments))).isFile();
        }
        catch (err) {
            return false;
        }
    });
}
exports.hasFile = hasFile;
function asyncForEach(array, callback) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        for (let index = 0; index < array.length; index++) {
            yield callback(array[index], index, array);
        }
    });
}
function collectTab({ cwd, packagePath }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const packages = {};
        const configs = [];
        const packageData = yield read_pkg_1.default({
            cwd,
            normalize: false,
        });
        const hasScript = (scriptName) => packageData.scripts && packageData.scripts[scriptName] && packageData.scripts[scriptName] !== '';
        const npmScriptsCmds = [
            hasScript('bootstrap') ? 'npm run bootstrap' : undefined,
            hasScript('build') ? 'npm run build' : undefined,
            hasScript('watch') ? 'npm run watch' : undefined,
            hasScript('test') ? 'npm run test' : undefined,
            hasScript('lint') ? 'npm run lint' : undefined,
            hasScript('clean') ? 'npm run clean' : undefined,
            hasScript('flow') ? 'npm run flow' : undefined,
            hasScript('compile') ? 'npm run compile' : undefined,
            hasScript('format') ? 'npm run format' : undefined,
            hasScript('prettier') ? 'npm run prettier' : undefined,
            hasScript('coverage') ? 'npm run coverage' : undefined,
            hasScript('prepare') ? 'npm run prepare' : undefined,
            hasScript('validate') ? 'npm run validate' : undefined,
            hasScript('deploy') ? 'npm run deploy' : undefined,
            hasScript('docs') ? 'npm run docs' : undefined,
            hasScript('build-storybook') ? 'npm run build-storybook' : undefined,
        ].filter((cmd) => cmd);
        if (hasScript('start')) {
            configs.push({
                pluginName: 'PluginTerminal',
                options: {
                    cmd: 'npm start',
                    cwd: packagePath,
                    autostart: true,
                },
            });
            packages['@dash4/plugin-terminal'] = true;
        }
        if (hasScript('storybook')) {
            configs.push({
                pluginName: 'PluginTerminal',
                options: {
                    cmd: 'npm run storybook',
                    cwd: packagePath,
                },
            });
            packages['@dash4/plugin-terminal'] = true;
        }
        const hasReadmeLow = yield hasFile(cwd, 'readme.md');
        const hasReadmeUp = yield hasFile(cwd, 'README.md');
        const hasReadme = hasReadmeLow || hasReadmeUp;
        if (hasReadme) {
            configs.push({
                pluginName: 'PluginReadme',
                options: {
                    file: hasReadmeUp
                        ? path_1.default.join(packagePath || '', 'README.md')
                        : path_1.default.join(packagePath || '', 'readme.md'),
                },
            });
            packages['@dash4/plugin-readme'] = true;
        }
        const hasWatchTest = hasScript('watch-test');
        if (hasScript('test') || hasWatchTest) {
            if (hasWatchTest) {
                configs.push({
                    pluginName: 'PluginTerminal',
                    options: {
                        cmd: 'npm run watch-test',
                        cwd: packagePath,
                    },
                });
                packages['@dash4/plugin-terminal'] = true;
            }
            configs.push(Object.assign({ pluginName: 'PluginCodeCoverage' }, (packagePath
                ? {
                    options: {
                        cwd: packagePath,
                    },
                }
                : {})));
            packages['@dash4/plugin-code-coverage'] = true;
        }
        if (npmScriptsCmds.length > 0) {
            configs.push({
                pluginName: 'PluginNpmScripts',
                options: {
                    scripts: npmScriptsCmds.map((cmd) => ({
                        title: cmd ? cmd.replace(/^npm run /, '') : '',
                        cmd,
                    })),
                    cwd: packagePath,
                },
            });
            packages['@dash4/plugin-npm-scripts'] = true;
        }
        return { packages, configs };
    });
}
function init(cwd, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const spin = log_1.spinner('cli', 'Collect required changes');
        spin.start();
        if (!(yield hasFile(cwd, 'package.json'))) {
            spin.fail('package.json not found!');
            process.kill(1);
            return;
        }
        if ((yield hasFile(options.config)) && !options.force) {
            spin.fail('Dash4 is already installed!');
            process.kill(1);
            return;
        }
        const config = new Config_1.Config({ port: options.port });
        let packages = { '@dash4/server': true };
        const packageData = yield read_pkg_1.default({ cwd, normalize: false });
        const collection = yield collectTab({
            cwd,
        });
        packages = Object.assign({}, packages, collection.packages);
        collection.configs.forEach((_config) => {
            config.addPlugin('Root', _config.pluginName, _config.options);
        });
        // lerna
        if (yield hasFile(cwd, 'lerna.json')) {
            spin.text('collect changes for lerna monorepository');
            const lernaPackages = yield get_lerna_packages_1.getLernaPackages(cwd);
            asyncForEach(lernaPackages, (lernaPackage) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const tabName = (yield fs_extra_1.default.readJSON(path_1.default.join(cwd, lernaPackage, 'package.json'))).name || path_1.default.basename(lernaPackage);
                const _collection = yield collectTab({
                    cwd: path_1.default.join(cwd, lernaPackage),
                    packagePath: lernaPackage,
                });
                packages = Object.assign({}, packages, _collection.packages);
                _collection.configs.forEach((_config) => {
                    config.addPlugin(tabName, _config.pluginName, _config.options);
                });
            }));
        }
        // add npm scripts
        spin.text('add dash4 npm script');
        packageData.scripts = packageData.scripts || {};
        packageData.scripts.dash4 = 'dash4';
        yield write_pkg_1.default(path_1.default.join(cwd, 'package.json'), packageData);
        // write dash.config
        spin.text('create dash4 configuration');
        yield fs_extra_1.default.writeFile(options.config, config.toString());
        // install dependencies
        spin.text('install dependencies');
        try {
            yield execa_1.default('npm', ['i', '-D', ...Object.keys(packages)]);
        }
        catch (err) {
            spin.fail(`${chalk_1.default.bold('Could not install dependencies:')}\n${err}`);
            process.kill(1);
        }
        spin.succeed('your Dash4 dashboard is installed and ready. Run "npm run dash4"');
    });
}
exports.init = init;
//# sourceMappingURL=index.js.map